import time
from typing import Optional

from . import client

class Tracker():
    def __init__(self, markingPeriod : int, schoolName : Optional[str] = None, email : Optional[str] = None, ID : Optional[int] = None, password : Optional[str] = None):
        self.markingPeriod : int = markingPeriod
        self.client = client.Client(False, schoolName, email, ID, password)
        self.grades = self.client.getGrades()[self.markingPeriod-1]["Data"]
    
    def track(self) -> None:
        count : int = 1
        newGrades : list
        while True:
            newGrades = self.client.getGrades()[self.markingPeriod-1]["Data"]
            if (newGrades != self.grades):
                print("Grades have changed!\a\a\a")
                self.grades = newGrades
            time.sleep(30)



from setuptools import setup, find_packages
import re
import os

def get_version(package):
    with open(os.path.join(package, "__init__.py")) as f:
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", f.read()).group(1)

def get_long_description():
    with open("README.md", encoding="utf8") as f:
        return f.read()

setup(
    name="plusportals",
    version=get_version("plusportals"),
    url="https://github.com/DhruvBisla/PlusPortalsAPI",
    license="MIT",
    author="Dhruv Bisla",
    author_email="bisladhruv@gmail.com",
    description="A reverse engineered API PlusPortals",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    keywords=['api', 'reverse-engineered', 'plusportals', 'plusportalsapi'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    requires = ['requests','lxml'],
    install_requires=['requests','lxml'],
    zip_safe=False,
)



import requests
from lxml import html
import json
from typing import List

from . import info

class Session():
    def __init__(self, schoolName, email, password):
        self.createSession()
        self.__schoolName : str = schoolName
        self.__email : str = email
        self.__password : str = password
        
        self.hasGetLanding : bool = False
        self.hasLogin : bool = False
        self.hasGetDetails : bool = False
        self.hasGetMarkingPeriod : bool = False
        self.requestVerificationToken : str = None
        
    def createSession(self) -> None:
        self.session : requests.Session = requests.Session()

    def getLanding(self) -> requests.Response:
        response = self.session.get(info.LANDING_LOGIN(self.__schoolName))
        self.hasGetLanding = True
        return response

    def login(self) -> requests.Response:
        None if self.hasGetLanding else self.getLanding()
        data = [
            ('UserName', self.__email),
            ('Password', self.__password),
            ('RememberMe', 'true'),
            ('btnsumit', 'Sign In'),
        ]
        specHeaders = {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'cookie': '__cfduid={}; ppschoollink={}; UGUID={}; __RequestVerificationToken={}; _pps=-480'.format(self.session.cookies.get_dict().get("__cfduid"), self.__schoolName, self.session.cookies.get_dict().get("UGUID"), self.session.cookies.get_dict().get("__RequestVerificationToken"))
        }
        response = self.session.post(info.LANDING_LOGIN(self.__schoolName), headers=dict(info.BASE_HEADERS, **specHeaders), data=data)
        self.hasLogin = True
        return response

    def getDetails(self) -> requests.Response:
        None if self.hasLogin else self.login()
        specHeaders = {
            'cookie': '__cfduid={}; ppschoollink={}; __RequestVerificationToken={}; _pps=-480; ASP.NET_SessionId={}; emailoption=RecentEmails; UGUID={}; ppusername={}; .ASPXAUTH={}'.format(self.session.cookies.get_dict().get('__cfduid'), self.__schoolName, self.session.cookies.get_dict().get('__RequestVerificationToken'), self.session.cookies.get_dict().get('ASP.NET_SessionId'), self.session.cookies.get_dict().get('UGUID'), self.session.cookies.get_dict().get('ppusername'), self.session.cookies.get_dict().get('.ASPXAUTH'))
        }
        response = self.session.post(info.DETAILS(self.__schoolName), headers=dict(info.BASE_HEADERS, **specHeaders))
        tree = html.fromstring(response.text)
        
        try:
            self.requestVerificationToken = tree.xpath("/html/body/input/@value")[0]
        except Exception as error:
            raise Exception("Login was not successful")
        finally:
            self.hasGetDetails = True
            return response

    def getMarkingPeriods(self) -> List[int]:
        None if self.hasGetDetails else self.getDetails()
        specHeaders = {
            '__requestverificationtoken': '{}'.format(self.requestVerificationToken),
            'cookie': '__cfduid={}; ppschoollink={}; __RequestVerificationToken={}; _pps=-480; ASP.NET_SessionId={}; emailoption={}; UGUID={}; ppusername={}; .ASPXAUTH={}'.format(self.session.cookies.get_dict().get('__cfduid'), self.__schoolName, self.session.cookies.get_dict().get('__RequestVerificationToken'), self.session.cookies.get_dict().get('ASP.NET_SessionId'), self.session.cookies.get_dict().get('emailoption'), self.session.cookies.get_dict().get('UGUID'), self.session.cookies.get_dict().get('ppusername'), self.session.cookies.get_dict().get('.ASPXAUTH'))
        }
        response = self.session.post(info.MARKING_PERIOD, headers=dict(info.BASE_HEADERS, **specHeaders))
        markingPeriods : List[int] = []
        try:
            markingDict = json.loads(response.content.decode('utf-8'))
            for period in markingDict:
                markingPeriods.append(period["MarkingPeriodId"])
            markingPeriods = markingPeriods[1:]
        except Exception as error:
            raise Exception("Invalid marking period, response returned: {}".format(error))
        finally:
            return markingPeriods


import os
import json
import requests
from typing import Optional

from . import credentials
from . import info
from . import session

class Client(session.Session):
    _SCHOOL_NAME : str = None
    _EMAIL : str = None
    _ID : int = None
    _PASSWORD : str = None
    markingPeriods : list = []
    hasCachedCredentials : bool = (os.path.isfile(os.path.join((os.path.dirname(__file__)), 'credentials.json')))

    def __init__(self, cacheCredentials : Optional[bool] = False, schoolName : Optional[str] = None, email : Optional[str] = None, ID : Optional[int] = None, password : Optional[str] = None):
        if cacheCredentials: Client.setCredentials(schoolName, email, ID, password)
        else:
            Client._SCHOOL_NAME = schoolName
            Client._EMAIL = email
            Client._ID = ID
            Client._PASSWORD = password
        if (Client.hasCachedCredentials and password == None):
            Client._SCHOOL_NAME = credentials.getCredential('schoolName')
            Client._EMAIL = credentials.getCredential('email')
            Client._ID = credentials.getCredential('ID')
            Client._PASSWORD = credentials.getCredential('password')
        super().__init__(Client._SCHOOL_NAME, Client._EMAIL, Client._PASSWORD)
        Client.markingPeriods = self.getMarkingPeriods()
        self.hasFetchGrades : bool = False
        self.grades : list[dict] = []

    def reset(self) -> None:
        self.session.cookies.clear()
        self.getDetails()

    @classmethod
    def setCredentials(cls, schoolName: str, email: str, ID: int, password: str) -> None:
        credentials.setCredentials(schoolName, email, ID, password)
        Client.hasCachedCredentials = True
    
    def fetchGrades(self) -> requests.Response:
        None if (Client.markingPeriods is not None) else self.getMarkingPeriods()
        specHeaders = {
            '__requestverificationtoken': '{}'.format(self.requestVerificationToken),
            'cookie': '__cfduid={}; ppschoollink={}; __RequestVerificationToken={}; _pps=-480; ASP.NET_SessionId={}; emailoption={}; UGUID={}; ppusername={}; .ASPXAUTH={}'.format(self.session.cookies.get_dict().get('__cfduid'), Client._SCHOOL_NAME, self.session.cookies.get_dict().get('__RequestVerificationToken'), self.session.cookies.get_dict().get('ASP.NET_SessionId'), self.session.cookies.get_dict().get('emailoption'), self.session.cookies.get_dict().get('UGUID'), self.session.cookies.get_dict().get('ppusername'), self.session.cookies.get_dict().get('.ASPXAUTH'))
        }
        try:
            agrades : list[dict] = []
            responses : list[requests.Response.status_code] = []
            for i in range(len(Client.markingPeriods)):
                response = (self.session.post(info.GRADES(self.markingPeriods[i]), headers=dict(info.BASE_HEADERS, **specHeaders)))
                agrades.append(json.loads(response.content.decode('utf-8')))
                responses.append(response.status_code)
        except:
            print("Information provided was incorrect; Login was not successful.")
        self.grades = agrades
        self.hasFetchGrades = True
        return responses
    
    def getGrades(self) -> list:
        None if (self.hasFetchGrades) else self.fetchGrades()
        return self.grades

    def printGrades(self, markingPeriod : int) -> None:
        mgrades = self.getGrades[markingPeriod-1]
        for i in mgrades["Data"]:
            print("{}'s grade is {}".format(i.get("CourseName")[:(len(i.get("CourseName")))-12],i.get("Average")))